﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Xml.Linq;

using Jp.Co.Atmarkit.Sample.Dto;
using Jp.Co.Atmarkit.Sample.Entity;
using Jp.Co.Atmarkit.Sample.Event;
using Jp.Co.Atmarkit.Sample.Utils;
using System.Collections.ObjectModel;

namespace Jp.Co.Atmarkit.Sample.Dao
{
    /// <summary>
    /// ユーザー情報を取得するためのDAOです。
    /// </summary>
    public class UserDao
    {
        /// <summary>
        /// デリゲートの宣言：ResultEventArgs型のオブジェクトを返すようにする
        /// </summary>
        /// <param name="sender">イベントのソースです。</param>
        /// <param name="e">イベント データを格納している EventArgsです。</param>
        public delegate void ResultEventHandler(object sender, ResultEventArgs e);

        /// <summary>
        /// イベントデリゲートです。
        /// </summary>
        public event ResultEventHandler Result;

        /// <summary>
        /// ユーザー一覧を取得します。
        /// ※実際はこのクラスを経由してサーバーからユーザー一覧を取得します。
        /// </summary>
        /// <param name="dto">リクエストDTOです。</param>
        public void GetUserList(UserRequestDto dto)
        {
            DebugUtil.Log("UserDao#GetUserList");
            GetUserListResultHandler();
        }

        /// <summary>
        /// ユーザー一覧取得の結果イベントハンドラです。
        /// </summary>
        private void GetUserListResultHandler()
        {
            DebugUtil.Log("UserDao#GetUserListResultHandler");

            // ユーザー一覧が取得できたと仮定して、結果を保持するレスポンスDTOを作成します。
            UserResponseDto responseDto = new UserResponseDto();

            // XMLからユーザー一覧を生成し、レスポンスDTOにセットします。
            responseDto.UserList = CreateUserListFromXML();

            ResultEventArgs args = new ResultEventArgs();

            args.Result = responseDto;

            Result(this, args);
        }

        /// <summary>
        /// XMLからユーザー一覧を作成します。
        /// </summary>
        /// <returns>ユーザー一覧です。</returns>
        private List<User> CreateUserListFromXML()
        {
            // リソースファイルを取得します。
            System.Reflection.Assembly asm = this.GetType().Assembly;
            System.Resources.ResourceManager rm = new System.Resources.ResourceManager("AtmarkitSilverlightSample.Resource1", asm);

            // user.xmlファイルの内容を文字列として取り出します。
            String s = rm.GetString("users");
            
            // 文字列ベースでXMLドキュメントを生成します。
            XDocument users = XDocument.Parse(s);

            // LINQで全件検索します
            var userList = from user 
                               in users.Descendants("user")
                               select user;

            List<User> resultValue = new List<User>();

            // XMLの配列をオブジェクトの配列に変換します。
            foreach(var user in userList)
            {
                resultValue.Add(User.Populate(user));
            }

            return resultValue;
        }
    }
}
